# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify Alma, CentOS, CentOS Stream and Rocky Linux as rhel
%if "%{__distinit}" == "a" || "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "cs" || "%{__distinit}" == "rl"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

Name:		perl-Eval-Closure
Version:	0.14
Release:	20.%{__distinit}%{__distvers}
Summary:	Safely and cleanly create closures via string eval
License:	GPL-1.0-or-later OR Artistic-1.0-Perl
URL:		https://metacpan.org/release/Eval-Closure
Source0:	https://cpan.metacpan.org/authors/id/D/DO/DOY/Eval-Closure-%{version}.tar.gz
BuildArch:	noarch
# Module Build
BuildRequires:	coreutils
BuildRequires:	findutils
BuildRequires:	make
BuildRequires:	perl-generators
BuildRequires:	perl-interpreter
BuildRequires:	perl(ExtUtils::MakeMaker)
# Module Runtime
BuildRequires:	perl(Carp)
BuildRequires:	perl(constant)
BuildRequires:	perl(Devel::LexAlias) >= 0.05
BuildRequires:	perl(Exporter)
BuildRequires:	perl(overload)
BuildRequires:	perl(Perl::Tidy)
BuildRequires:	perl(Scalar::Util)
BuildRequires:	perl(strict)
# Test Suite
BuildRequires:	perl(B)
BuildRequires:	perl(blib)
BuildRequires:	perl(File::Spec)
BuildRequires:	perl(IO::Handle)
BuildRequires:	perl(IPC::Open3)
BuildRequires:	perl(PadWalker)
BuildRequires:	perl(Test::Fatal)
BuildRequires:	perl(Test::More) >= 0.88
BuildRequires:	perl(Test::Requires)
BuildRequires:	perl(warnings)
# Optional Tests
BuildRequires:	perl(Test::Output)
# Extra Tests
BuildRequires:	perl(Pod::Coverage::TrustPod)
BuildRequires:	perl(Test::EOL)
BuildRequires:	perl(Test::NoTabs)
BuildRequires:	perl(Test::Pod::Coverage) >= 1.08
BuildRequires:	perl(Test::Pod) >= 1.41
# Dependencies
%if 0%{?fedora} < 38 && 0%{?rhel} < 10
Requires:	perl(:MODULE_COMPAT_%(eval "`perl -V:version`"; echo $version))
%endif
Requires:	perl(Devel::LexAlias) >= 0.05
Requires:	perl(Perl::Tidy)

%description
String eval is often used for dynamic code generation. For instance, Moose uses
it heavily, to generate inlined versions of accessors and constructors, which
speeds code up at runtime by a significant amount.  String eval is not without
its issues however - it's difficult to control the scope it's used in (which
determines which variables are in scope inside the eval), and it can be quite
slow, especially if doing a large number of evals.

This module attempts to solve both of those problems. It provides an
eval_closure function, which evals a string in a clean environment, other than
a fixed list of specified variables. It also caches the result of the eval, so
that doing repeated evals of the same source, even with a different
environment, will be much faster (but note that the description is part of the
string to be evaled, so it must also be the same (or non-existent) if caching
is to work properly).

%prep
%setup -q -n Eval-Closure-%{version}

%build
perl Makefile.PL INSTALLDIRS=vendor
make %{?_smp_mflags}

%install
make pure_install DESTDIR=%{buildroot}
find %{buildroot} -type f -name .packlist -delete
%{_fixperms} -c %{buildroot}

%check
make test
make test TEST_FILES="$(echo $(find xt/ -name '*.t'))"

%files
%license LICENSE
%doc Changes README
%{perl_vendorlib}/Eval/
%{_mandir}/man3/Eval::Closure.3*

%changelog
* Thu Apr  6 2023 Paul Howarth <paul@city-fan.org> - 0.14-20
- Use SPDX-format license tag
- Use %%license unconditionally

* Fri May 31 2019 Paul Howarth <paul@city-fan.org> - 0.14-9
- Modernize spec
  - Switch upstream from search.cpan.org to metacpan.org
  - BR: perl-generators unconditionally
  - BR: perl-interpreter rather than perl
  - Don't explicitly clean buildroot in %%install section
  - Drop explicit %%clean section
  - Drop support for building with Test::More < 0.88
  - Drop legacy BuildRoot: and Group: tags

* Sun May 29 2016 Paul Howarth <paul@city-fan.org> - 0.14-1
- Update to 0.14
  - A couple of minor optimizations (GH#8)
- BR: perl-generators where available
- Simplify find command using -delete
- Update patch for building with old Test::More versions

* Sat Jun 27 2015 Paul Howarth <paul@city-fan.org> - 0.13-3
- Perl 5.22 rebuild

* Tue May 12 2015 Paul Howarth <paul@city-fan.org> - 0.13-1
- Update to 0.13
  - One more fix for blead (GH#6)

* Mon Jan 19 2015 Paul Howarth <paul@city-fan.org> - 0.12-1
- Update to 0.12
  - Fix tests under blead (GH#4)
- Update patches

* Thu Sep 11 2014 Paul Howarth <paul@city-fan.org> - 0.11-2
- Drop support for building with ExtUtils::MakeMaker < 6.30
- Drop support for building without Test::Requires
- Drop %%defattr, redundant since rpm 4.4
- Use %%license where possible
- Classify buildreqs by usage

* Wed Jul 31 2013 Paul Howarth <paul@city-fan.org> - 0.11-1
- Update to 0.11
  - Add "alias => 1" option for making closure variables actually alias the
    closed over variables (so the variable referenced in the environment
    hashref will actually be updated by changes made in the closure)
- BR:/R: perl(Devel::LexAlias) ≥ 0.05
- Update patches

* Tue Jul 23 2013 Paul Howarth <paul@city-fan.org> - 0.10-2
- Perl 5.18 rebuild

* Thu Jul 11 2013 Paul Howarth <paul@city-fan.org> - 0.10-1
- Update to 0.10
  - Support lexical subs on 5.18+
  - Fix pod links
- Explicitly run the extra tests
- Update patches

* Sun Jul  8 2012 Paul Howarth <paul@city-fan.org> - 0.08-2
- BR: perl(Carp) and perl(File::Temp)
- Don't need to remove empty directories from the buildroot

* Fri Feb 10 2012 Paul Howarth <paul@city-fan.org> - 0.08-1
- Update to 0.08
  - Remove a double layer of string eval that was introduced in 0.07 as an
    intermediate step in figuring out the unique package thing - it's not
    necessary with the final implementation, and just makes things slower and
    hides errors

* Sat Feb  4 2012 Paul Howarth <paul@city-fan.org> - 0.07-1
- Update to 0.07
  - The given source is now evaled in a unique package for every eval_closure
    call (it always used to be evaled in the Eval::Closure package, which was
    especially buggy); this is to avoid issues where one eval_closure modifies
    the global environment (by, say, importing a function), which could mess
    up a later call
  - Unfortunately, this means that the memoization stuff no longer works, since
    it will result in memoized results using the original package, which
    defeats the purpose; I'm open to suggestions on how to safely reenable it
  - Clean up a few stray lexicals we were still closing over in the eval
- Update patches

* Fri Jun 24 2011 Paul Howarth <paul@city-fan.org> - 0.06-2
- Rebuild for perl 5.14.1 in Rawhide
- Fix dist tag for CentOS 6 and Scientific Linux

* Mon Jun  6 2011 Paul Howarth <paul@city-fan.org> - 0.06-1
- Update to 0.06
  - The dumped source from setting $ENV{EVAL_CLOSURE_PRINT_SOURCE} didn't show
    the captured variable creation properly
- Update patch for old ExtUtils::MakeMaker versions

* Tue Apr 26 2011 Paul Howarth <paul@city-fan.org> - 0.05-1
- Update to 0.05
  - Make sure Perl::Tidy doesn't look at @ARGV

* Mon Apr 18 2011 Paul Howarth <paul@city-fan.org> - 0.04-1
- Update to 0.04
  - Stop using Memoize, which apparently doesn't work properly under mod_perl
    in some situations

* Tue Apr 12 2011 Paul Howarth <paul@city-fan.org> - 0.03-2
- Import from Fedora

* Thu Mar  3 2011 Iain Arnell <iarnell@gmail.com> - 0.03-1
- Update to latest upstream version

* Tue Feb  8 2011 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.02-2
- rebuilt for https://fedoraproject.org/wiki/Fedora_15_Mass_Rebuild

* Fri Jan 28 2011 Iain Arnell <iarnell@gmail.com> - 0.02-1
- Update to latest upstream version

* Sun Jan 23 2011 Iain Arnell <iarnell@gmail.com> - 0.01-1
- Specfile autogenerated by cpanspec 1.78
